#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: generate_vserver_list.pl, v1.1.0.0 Thu 26 Jul 2007 08:21:13 PM CEST mstauber Exp $
# Copyright 2006-2007 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.

# This constructor / handler uses 'vzlist' to poll extended vserver status.
# Additionally it parses the vserver config files in /etc/sysconfig/vz-scripts/
# The information is then written off into CCE for GUI usage. 

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

$cce->connectfd();
#$cce->connectuds(); # only for debugging

# Location of the vzlist-dump:
$vzlistdump = '/tmp/vserver-list-short';
$vzcfgdir = '/etc/sysconfig/vz-scripts/';

# Generate vzlist-dump:
#system("vzlist -a -o vpsid,ip,hostname,status,numproc,laverage,cpuunits | sed 's/^[ \t]*//;s/[ \t]*\$//' | sed 's/-/0/'g > $vzlistdump");
system("vzlist -a -o vpsid,ip,hostname,status,numproc,laverage,cpuunits | sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzlistdump");

# Pull the vzlist-dump into a Matrix:
#
# I LOVE matrix stuff! If there is a sledload of data to parse, 
# then there is hardly a more efficient way to do so.
# A legend of what colums the matrix has is at the end of the code.

open (F, $vzlistdump) || die "Could not open $vzlistdump: $!";
$a = 0; 				# Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
while ($line = <F>) {
    chomp($line);
    next if $line =~ /^\s*$/; 		# skip blank lines
    if ($line =~ /^([A-Za-z]\w*)/) {
        #$matrix_name = $1;
        $matrix_name = 'VPSID';
    } 
    else {
        my (@row) = split (/\s+/, $line);
        push (@{$matrix_name}, \@row); 
    }
    if ($VPSID[$a][0]) { 		# Plausibility check. We will not store anything into CCE if 'vpsid' is not set.
	$currentvpsid = $VPSID[$a][0];
	&feedthemonster;		# Feed info into CCE
	if ($new_vps_found == '1') {
	    &feedthemonster;		# We found a new VPS. We need to feed the monster again!
	}
    }
    $a++;
}
close(F);
system("rm $vzlistdump");

# Subroutine that feeds the data into CCE:
sub feedthemonster {
	&readconfig;
	&validate;
	# This is where we actually populate CCE with the gathered info:
        @oids = $cce->find('Vserver', {'vpsid' => $currentvpsid});
        if ($#oids < 0) {
                #print "Object not yet in CCE. Creating new one.\n";
                ($ok) = $cce->create('Vserver', {
                'vpsid' => $VPSID[$a][0],
                'ip' => $VPSID[$a][1],
                'hostname' => $VPSID[$a][2],
		'ONBOOT' => $VPSID[$a][114],
		'name' => 'admin',
                'timestamp' => time()
                });
		$new_vps_found = '1'; 			# We found a new VPS, one that we have never seen before and which
							# is not yet in CCE. We do not want to risk to populate CCE with ALL
							# the found new data in one go. So let us just set the most basic info
							# and the timestamp. 
							#
							# We also set $new_vps_found = '1' to run feedthemonster() again.
        }
        else {
	        #print "Object already present in CCE. Updating it.\n";
                # find the system object
		$new_vps_found = '0';
		&validate;
                ($sys_oid) = $cce->find('Vserver', {'vpsid' => $currentvpsid});
                ($ok, $sys) = $cce->get($sys_oid);
		
		# For debugging and logging purpose we do not write all 122 entries into CCE in one go. Instead we break it down
		# into multiple blocks so that we can troubleshoot /var/log/messages easily for data-blocks with values that CCE
		# did not like. 
		
                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'vpsid' => $VPSID[$a][0],
                'ip' => $VPSID[$a][1],
                'hostname' => $VPSID[$a][2],
                'status' => $VPSID[$a][3],
                'numproc' => $VPSID[$a][4],
                'laverage' => $VPSID[$a][5],
                'cpuunits' => $VPSID[$a][6],
                'ONBOOT' => $VPSID[$a][114],
                'name' => 'admin',
                'timestamp' => time()
                });
	}

}


# Validate the settings before we pass them on to CCE:
sub validate {

    # ip:
    if ($VPSID[$a][1] == 0) {
	$VPSID[$a][1] = "127.0.0.1";
    }
    # hostname:
    if (!$VPSID[$a][2]) {
	$VPSID[$a][2] = "localhost.localdomain";
    }

    # numproc:
    if ($VPSID[$a][4] eq "-") {
	$VPSID[$a][4] = "0";
    }
    # laverage:
    if ($VPSID[$a][5] eq "-") {
	$VPSID[$a][5] = "0";
    }
}

sub readconfig {

    # We need to parse the config file for the onboot switch:
    $cfile = "$vzcfgdir$currentvpsid.conf";
    if (-e $cfile) {
        open(CFILE,"$cfile") || die("Could not find $cfile. $!");
        while ($line = <CFILE>) {
            chomp($line);
            next if $line =~ /^\s*$/; # skip blank lines

            if ($line =~ /^ONBOOT=\"([A-Za-z]\w*)\"/) {
                $onboot = $1;
                if (($onboot eq 'yes') || ($onboot eq 'Yes')) {
                    $VPSID[$a][114] = 'Yes';
                }
                else {
                    $VPSID[$a][114] = 'No';
                }
	    }
	}
    }
}

$cce->bye('SUCCESS');
exit(0);

